@@ -15,7 +15,7 @@ use POSIX qw(ULONG_MAX LONG_MIN);
 
 use Class::Multimethods qw();
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 our ($ROUND, $PREC);
 
 BEGIN {
@@ -281,14 +281,16 @@ use overload
 
             if ($name eq ':overload') {
                 overload::constant
-                  integer => sub { __PACKAGE__->new_ui($_[0]) },
-                  float   => sub { __PACKAGE__->new_f($_[0]) },
+                  integer => sub { bless \Math::GMPz::Rmpz_init_set_ui($_[0]) },
+                  float   => sub { bless \_str2obj($_[0]) },
                   binary  => sub {
-                    my ($const) = @_;
+                    my $const = ($_[0] =~ tr/_//dr);
                     my $prefix = substr($const, 0, 2);
-                        $prefix eq '0x' ? __PACKAGE__->new(substr($const, 2), 16)
-                      : $prefix eq '0b' ? __PACKAGE__->new(substr($const, 2), 2)
-                      :                   __PACKAGE__->new(substr($const, 1), 8);
+                    bless \(
+                              $prefix eq '0x' ? Math::GMPz::Rmpz_init_set_str(substr($const, 2) || 0, 16)
+                            : $prefix eq '0b' ? Math::GMPz::Rmpz_init_set_str(substr($const, 2) || 0, 2)
+                            :                   Math::GMPz::Rmpz_init_set_str(substr($const, 1) || 0, 8)
+                           );
                   };
 
                 # Export 'Inf', 'NaN' and 'i' as constants
@@ -782,13 +784,6 @@ sub new {
         return bless(\Math::GMPz::Rmpz_init_set($num), $class);
     }
 
-    # BigNum
-    elsif ($ref eq 'Math::BigNum') {
-        my $r = Math::GMPq::Rmpq_init();
-        Math::GMPq::Rmpq_set($r, $$num);
-        return bless \$r, $class;
-    }
-
     # MPFR
     elsif ($ref eq 'Math::MPFR') {
         my $r = Math::MPFR::Rmpfr_init2($PREC);
@@ -862,20 +857,17 @@ sub new {
 
 sub new_si {
     my ($class, $si) = @_;
-    my $r = Math::GMPz::Rmpz_init_set_si($si);
-    bless \$r, $class;
+    bless \Math::GMPz::Rmpz_init_set_si($si), $class;
 }
 
 sub new_ui {
     my ($class, $ui) = @_;
-    my $r = Math::GMPz::Rmpz_init_set_ui($ui);
-    bless \$r, $class;
+    bless \Math::GMPz::Rmpz_init_set_ui($ui), $class;
 }
 
 sub new_z {
     my ($class, $str, $base) = @_;
-    my $r = Math::GMPz::Rmpz_init_set_str($str, $base // 10);
-    bless \$r, $class;
+    bless \Math::GMPz::Rmpz_init_set_str($str, $base // 10), $class;
 }
 
 sub new_q {
@@ -1607,14 +1599,6 @@ Class::Multimethods::multimethod div => qw(Math::AnyNum $) => sub {
             Math::GMPq::Rmpq_mul($r, $r, $$x);
             return bless \$r;
         }
-        elsif (ref($$x) eq 'Math::GMPz') {
-            my $r = Math::GMPq::Rmpq_init();
-            Math::GMPq::Rmpq_set_ui($r, 1, CORE::abs($y));
-            Math::GMPq::Rmpq_set_num($r, $$x);
-            Math::GMPq::Rmpq_neg($r, $r) if $y < 0;
-            Math::GMPq::Rmpq_canonicalize($r);
-            return bless \$r;
-        }
 
         bless \__div__(_copy($$x), $y);
     }
@@ -1884,18 +1868,22 @@ Class::Multimethods::multimethod root => qw(Math::AnyNum $) => sub {
     bless \__pow__(_copy($$x), __inv__(_str2obj($y)));
 };
 
+Class::Multimethods::multimethod root => qw($ $) => sub {
+    require Math::AnyNum::pow;
+    require Math::AnyNum::inv;
+    bless \__pow__(_str2obj($_[0]), __inv__(_str2obj($_[1])));
+};
+
 Class::Multimethods::multimethod root => qw(* $) => sub {
     require Math::AnyNum::pow;
     require Math::AnyNum::inv;
-    my ($x, $y) = @_;
-    bless \__pow__(${__PACKAGE__->new($x)}, __inv__(_str2obj($y)));
+    bless \__pow__(${__PACKAGE__->new($_[0])}, __inv__(_str2obj($_[1])));
 };
 
 Class::Multimethods::multimethod root => qw(* *) => sub {
     require Math::AnyNum::pow;
     require Math::AnyNum::inv;
-    my ($x, $y) = @_;
-    bless \__pow__(${__PACKAGE__->new($x)}, __inv__(${__PACKAGE__->new($y)}));
+    bless \__pow__(${__PACKAGE__->new($_[0])}, __inv__(${__PACKAGE__->new($_[1])}));
 };
 
 #
@@ -2438,7 +2426,15 @@ sub digamma {
 #
 
 sub zeta {
-    my $r = _star2mpfr($_[0]);
+    my ($x) = @_;
+
+    if (!ref($x) and CORE::int($x) eq $x and $x >= 0 and $x <= ULONG_MAX) {
+        my $r = Math::MPFR::Rmpfr_init2($PREC);
+        Math::MPFR::Rmpfr_zeta_ui($r, $x, $ROUND);
+        return bless \$r;
+    }
+
+    my $r = _star2mpfr($x);
     Math::MPFR::Rmpfr_zeta($r, $r, $ROUND);
     bless \$r;
 }
@@ -2953,6 +2949,17 @@ Class::Multimethods::multimethod gcd => qw(Math::AnyNum Math::AnyNum) => sub {
     bless \__gcd__(_copy2mpz($$x) // (goto &nan), _any2mpz($$y) // (goto &nan));
 };
 
+Class::Multimethods::multimethod gcd => qw(Math::AnyNum $) => sub {
+    require Math::AnyNum::gcd;
+    my ($x, $y) = @_;
+    if (CORE::int($y) eq $y and CORE::abs($y) <= ULONG_MAX) {
+        bless \__gcd__(_copy2mpz($$x) // (goto &nan), CORE::abs($y));
+    }
+    else {
+        bless \__gcd__(_copy2mpz($$x) // (goto &nan), _star2mpz($y) // (goto &nan));
+    }
+};
+
 Class::Multimethods::multimethod gcd => qw(Math::AnyNum *) => sub {
     require Math::AnyNum::gcd;
     my ($x, $y) = @_;
@@ -2965,6 +2972,18 @@ Class::Multimethods::multimethod gcd => qw(* Math::AnyNum) => sub {
     bless \__gcd__(_star2mpz($x) // (goto &nan), _any2mpz($$y) // (goto &nan));
 };
 
+Class::Multimethods::multimethod gcd => qw(* $) => sub {
+    require Math::AnyNum::gcd;
+    my ($x, $y) = @_;
+
+    if (CORE::int($y) eq $y and CORE::abs($y) <= ULONG_MAX) {
+        bless \__gcd__(_star2mpz($x) // (goto &nan), CORE::abs($y));
+    }
+    else {
+        bless \__gcd__(_star2mpz($x) // (goto &nan), _star2mpz($y) // (goto &nan));
+    }
+};
+
 Class::Multimethods::multimethod gcd => qw(* *) => sub {
     require Math::AnyNum::gcd;
     my ($x, $y) = @_;
@@ -2987,12 +3006,36 @@ Class::Multimethods::multimethod lcm => qw(Math::AnyNum *) => sub {
     bless \__lcm__(_copy2mpz($$x) // (goto &nan), _star2mpz($y) // (goto &nan));
 };
 
+Class::Multimethods::multimethod lcm => qw(Math::AnyNum $) => sub {
+    require Math::AnyNum::lcm;
+    my ($x, $y) = @_;
+
+    if (CORE::int($y) eq $y and CORE::abs($y) <= ULONG_MAX) {
+        bless \__lcm__(_copy2mpz($$x) // (goto &nan), CORE::abs($y));
+    }
+    else {
+        bless \__lcm__(_copy2mpz($$x) // (goto &nan), _star2mpz($y) // (goto &nan));
+    }
+};
+
 Class::Multimethods::multimethod lcm => qw(* Math::AnyNum) => sub {
     require Math::AnyNum::lcm;
     my ($x, $y) = @_;
     bless \__lcm__(_star2mpz($x) // (goto &nan), _any2mpz($$y) // (goto &nan));
 };
 
+Class::Multimethods::multimethod lcm => qw(* $) => sub {
+    require Math::AnyNum::lcm;
+    my ($x, $y) = @_;
+
+    if (CORE::int($y) eq $y and CORE::abs($y) <= ULONG_MAX) {
+        bless \__lcm__(_star2mpz($x) // (goto &nan), CORE::abs($y));
+    }
+    else {
+        bless \__lcm__(_star2mpz($x) // (goto &nan), _star2mpz($y) // (goto &nan));
+    }
+};
+
 Class::Multimethods::multimethod lcm => qw(* *) => sub {
     require Math::AnyNum::lcm;
     bless \__lcm__(_star2mpz($_[0]) // (goto &nan), _star2mpz($_[1]) // (goto &nan));
