@@ -60,6 +60,12 @@ Class::Multimethods::multimethod __div__ => qw(Math::GMPz Math::GMPz) => sub {
         goto &__div__;
     };
 
+    # Check for exact divisibility
+    if (Math::GMPz::Rmpz_divisible_p($x, $y)) {
+        Math::GMPz::Rmpz_divexact($x, $x, $y);
+        return $x;
+    }
+
     my $r = Math::GMPq::Rmpq_init();
     Math::GMPq::Rmpq_set_num($r, $x);
     Math::GMPq::Rmpq_set_den($r, $y);
@@ -67,6 +73,24 @@ Class::Multimethods::multimethod __div__ => qw(Math::GMPz Math::GMPz) => sub {
     $r;
 };
 
+Class::Multimethods::multimethod __div__ => qw(Math::GMPz $) => sub {
+    my ($x, $y) = @_;
+
+    # Check for exact divisibility
+    if (Math::GMPz::Rmpz_divisible_ui_p($x, CORE::abs($y))) {
+        Math::GMPz::Rmpz_divexact_ui($x, $x, CORE::abs($y));
+        Math::GMPz::Rmpz_neg($x, $x) if $y < 0;
+        return $x;
+    }
+
+    my $r = Math::GMPq::Rmpq_init();
+    Math::GMPq::Rmpq_set_ui($r, 1, CORE::abs($y));
+    Math::GMPq::Rmpq_set_num($r, $x);
+    Math::GMPq::Rmpq_neg($r, $r) if $y < 0;
+    Math::GMPq::Rmpq_canonicalize($r);
+    $r;
+};
+
 Class::Multimethods::multimethod __div__ => qw(Math::GMPz Math::GMPq) => sub {
     my ($x, $y) = @_;
 
@@ -76,8 +100,6 @@ Class::Multimethods::multimethod __div__ => qw(Math::GMPz Math::GMPq) => sub {
         goto &__div__;
     };
 
-    Math::GMPz::Rmpz_sgn($x) || return $x;
-
     my $q = Math::GMPq::Rmpq_init();
     Math::GMPq::Rmpq_z_div($q, $x, $y);
     $q;
